The user facing connector documentation should follow the guidelines defined in the [standard template](https://github.com/airbytehq/airbyte/blob/master/airbyte-ci/connectors/connectors_qa/src/connectors_qa/checks/documentation/templates/template.md.j2).

Check verifies that {{ header }} header section content follows standard template:

````
{{ template }}
````
